import { AppPage } from "./app.po";
import { ListPage } from "./list.po";
import { CalculatorPage } from "./calculator.po";
import { Navbar } from "./navbar.po";
import { browser } from "protractor";
import { PreferencesPage } from "./preferences.po";
import { changeSelectValue, scrollPageToTop } from "./util.po";

/**
 * Clone calculators
 */
describe("ngHyd − clone a calculator", () => {
    let startPage: AppPage;
    let listPage: ListPage;
    let calcPage: CalculatorPage;
    let navbar: Navbar;
    let prefPage: PreferencesPage;

    beforeEach(() => {
        startPage = new AppPage();
        listPage = new ListPage();
        calcPage = new CalculatorPage();
    });

    beforeAll(async () => {
        prefPage = new PreferencesPage();
        navbar = new Navbar();
        // disable evil option "empty fields on module creation"
        await prefPage.navigateTo();
        await prefPage.disableEvilEmptyFields();
        await browser.sleep(200);
    });

    it("when cloning a calculator, the clone should have the same values for all parameters", async () => {
        await navbar.clickNewCalculatorButton();
        // 1. create target modules for linked parameter
        await listPage.clickMenuEntryForCalcType(3); // Régime uniforme
        await browser.sleep(500);
        const debitRU = calcPage.getInputById("calc_Q"); // "Débit" is calculated by default
        await calcPage.setParamMode(debitRU, "fix");
        await browser.sleep(500);

        await navbar.clickNewCalculatorButton();
        await listPage.clickMenuEntryForCalcType(4); // Courbe de remous
        await browser.sleep(500);

        // 2. create source module to clone
        await navbar.clickNewCalculatorButton();
        await listPage.clickMenuEntryForCalcType(2); // Section paramétrée
        await browser.sleep(500);

        // 3. change and store source parameter values
        const sourceValues = {
            k: 0.6,
            Ks: 42
        };
        await changeSelectValue(calcPage.getSelectById("select_section"), 3); // mode "parabolique"
        await calcPage.getInputById("k").clear();
        await calcPage.getInputById("k").sendKeys(sourceValues["k"]);
        await calcPage.getInputById("Ks").clear();
        await calcPage.getInputById("Ks").sendKeys(sourceValues["Ks"]);
        // link "Débit" to "Courbe de remous"
        const debitSP = calcPage.getInputById("Q");
        await calcPage.setParamMode(debitSP, "link");
        await browser.sleep(500);
        await changeSelectValue(calcPage.getSelectById("linked_Q"), 1); // "Courbe de remous"
        await browser.sleep(500);

        // otherwise clickCloneCalcButton() fails with "Element is not clickable at point"
        await scrollPageToTop();
        await calcPage.clickCloneCalcButton();
        await browser.sleep(500);

        // 4. check existence of the cloned module
        expect(await navbar.getAllCalculatorTabs().count()).toBe(4);
        await navbar.clickCalculatorTab(3); // n°3 should be the latest
        await browser.sleep(500);

        // 5. compare values
        Object.keys(sourceValues).forEach(async (k) => {
            const v = sourceValues[k];
            const displayedVal = await calcPage.getInputById(k).getAttribute("value");
            expect(displayedVal).toBe("" + v);
        });

        // @TODO check linked value (see above)
    });
});
