import { ListPage } from "./list.po";
import { browser, by } from "protractor";
import { CalculatorPage } from "./calculator.po";
import { PreferencesPage } from "./preferences.po";
import { Navbar } from "./navbar.po";
import { changeSelectValue } from "./util.po";

/**
 * Check that created/cloned structures have empty fields when
 * "empty fields on calculator creation" is enabled
 */
describe("Lechapt&Calmon - ", () => {
    let prefPage: PreferencesPage;
    let listPage: ListPage;
    let calcPage: CalculatorPage;
    let navBar: Navbar;

    beforeAll(() => {
        prefPage = new PreferencesPage();
        listPage = new ListPage();
        calcPage = new CalculatorPage();
        navBar = new Navbar();
        browser.manage().window().setPosition(2000, 30);
    });

    beforeEach(async () => {
        // disable evil option "empty fields on module creation"
        await prefPage.navigateTo();
        await browser.sleep(200);
        await prefPage.disableEvilEmptyFields();
        await browser.sleep(200);
    });

    async function setup() {
        // start page
        await navBar.clickNewCalculatorButton();
        await browser.sleep(200);

        // open Lechapt-Calmon (pressure loss) calculator
        await listPage.clickMenuEntryForCalcType(35);
        await browser.sleep(200);
    }

    it("when material is modified, results should change", async () => {
        await setup();

        // select last material type
        const materialSelect = calcPage.getSelectById("select_material");
        await changeSelectValue(materialSelect, 8);
        await browser.sleep(200);

        // run calculation
        const calcButton = calcPage.getCalculateButton();
        await calcButton.click();
        await browser.sleep(200);

        // store total pressure loss result
        const res1 = calcPage.getAllFixedResultsRows().get(4);
        const pl1 = await res1.all(by.css("td")).get(1).getText();

        // select first material type
        await changeSelectValue(materialSelect, 0);
        await browser.sleep(200);

        // run calculation
        await calcButton.click();
        await browser.sleep(200);

        // compare total pressure loss result with first calculaiotn
        const res2 = calcPage.getAllFixedResultsRows().get(4);
        const pl2 = await res2.all(by.css("td")).get(1).getText();

        expect(pl1).not.toEqual(pl2);
    });
});
