import { browser, by, element, ElementFinder } from "protractor";
import { changeSelectValue } from "./util.po";

export class PreferencesPage {
    navigateTo() {
        return browser.get("/#/setup");
    }

    getHeader1() {
        return element(by.css("h1"));
    }

    getLanguageSelect() {
        return element(by.css(`mat-select[data-testid="language-select"]`));
    }

    getEmptyFieldsCheckbox() {
        return element(by.css(`mat-checkbox#cb_emptyFields`));
    }

    getInputLabels() {
        return element.all(by.css("label.mat-form-field-label"));
    }

    getNumericFormFields() {
        return element.all(by.css(`mat-form-field[data-testclass="numeric-input"]`));
    }

    getInputForField(ff: ElementFinder) {
        return ff.element(by.css("input"));
    }

    /**
     * find input with given "name" attribute
     */
    getInputFromName(name: string) {
        const cssSelector: string = `input[name="${name}"]`;
        return element(by.css(cssSelector));
    }

    getErrorsForField(ff: ElementFinder) {
        return ff.element(by.css("mat-error"));
    }

    async changeLanguage(index: number) {
        const select = this.getLanguageSelect();
        await changeSelectValue(select, index);
    }

    async enableEvilEmptyFields() {
        const cb = this.getEmptyFieldsCheckbox();
        const underlyingCB = cb.element(by.css(`input.mat-checkbox-input`));
        if (!underlyingCB.isSelected()) {
            await cb.click();
        }
    }

    async disableEvilEmptyFields() {
        const cb = this.getEmptyFieldsCheckbox();
        const underlyingCB = cb.element(by.css(`input.mat-checkbox-input`));
        if (underlyingCB.isSelected()) {
            await cb.click();
        }
    }

    /**
     * enable/disable option "empty fields on module creation"
     * @param b true to enable "empty fields on module creation" option, false to disable it (fill fields with default values)
     */
    async setEmptyFields(b: boolean) {
        await this.navigateTo();
        await browser.sleep(200);

        if (b) {
            await this.enableEvilEmptyFields();
        }
        else {
            await this.disableEvilEmptyFields();
        }
        await browser.sleep(200);
    }

    async setIterationCount(n: number) {
        const input = this.getInputFromName("nmi");
        input.clear();
        await input.sendKeys(n.toString());
    }
}
