import { ListPage } from "./list.po";
import { CalculatorPage } from "./calculator.po";
import { browser } from "protractor";
import { Navbar } from "./navbar.po";
import { PreferencesPage } from "./preferences.po";
import { SideNav } from "./sidenav.po";
import { changeSelectValue } from "./util.po";

/**
 * Remous
 */
describe("ngHyd − remous", () => {
    let listPage: ListPage;
    let calcPage: CalculatorPage;
    let navBar: Navbar;
    let prefPage: PreferencesPage;
    let sidenav: SideNav;

    beforeEach(async () => {
        listPage = new ListPage();
        prefPage = new PreferencesPage();
        calcPage = new CalculatorPage();
        navBar = new Navbar();
        sidenav = new SideNav();
        // disable evil option "empty fields on module creation"
        await prefPage.navigateTo();
        await prefPage.disableEvilEmptyFields();
        await browser.sleep(200);
    });

    it("when leaving calculated remous page then coming back to it, logs should not be duplicated", async () => {
        await navBar.clickNewCalculatorButton();

        // 1. create new Remous
        await listPage.clickMenuEntryForCalcType(4);
        await browser.sleep(300);

        // 2. calculate it, there should be 6 messages in the log
        await calcPage.getCalculateButton().click();
        expect(await calcPage.nbLogEntries()).toBe(6);
        await browser.sleep(300);

        // 3. go to home page (simplest example)
        await navBar.clickNewCalculatorButton();
        await browser.sleep(300);

        // 4. go back to Remous
        await navBar.clickCalculatorTab(0);
        await browser.sleep(300);

        // 5. there should still be 6 messages in the log
        expect(await calcPage.nbLogEntries()).toBe(6);
    });

    it("Calculation with large bed width should run successfully", async () => {
        await navBar.clickNewCalculatorButton();

        // 1. create new Remous
        await listPage.clickMenuEntryForCalcType(4);
        await browser.sleep(300);

        // 2. Set to trapezoidal section with bank slope of 2m/m and 20 meter width bed
        await changeSelectValue(calcPage.getSelectById("select_section"), 2);
        await browser.sleep(300);
        await calcPage.getInputById("LargeurFond").clear();
        await browser.sleep(300);
        await calcPage.getInputById("LargeurFond").sendKeys("20");
        await calcPage.getInputById("Fruit").clear();
        await browser.sleep(300);
        await calcPage.getInputById("Fruit").sendKeys("2");

        // 3. Calculate, the calculation should succeed
        await calcPage.getCalculateButton().click();
        const hasResults = await calcPage.hasResults();
        expect(hasResults).toBe(true);
    });

    it("Check that calculation with dx >= reach length fails", async () => {
        // load backwater curve session file with Dx>=Long
        await navBar.clickMenuButton();
        await browser.sleep(500);
        await sidenav.clickLoadSessionButton();
        await browser.sleep(500);
        await sidenav.loadSessionFile("./session/session-remous-dx-bief.json");
        await browser.sleep(1000);

        // Calculate, the calculation should fail
        await calcPage.getCalculateButton().click();
        await browser.sleep(1000);

        // check error message in log
        expect(await calcPage.nbLogEntries()).toBe(1);

        // check there is no result
        expect(await calcPage.presentAndVisible("var-results")).toBe(false);
    });
});
