import { AppPage } from "./app.po";
import { ListPage } from "./list.po";
import { CalculatorPage } from "./calculator.po";
import { Navbar } from "./navbar.po";
import { browser, by, element } from "protractor";
import { SideNav } from "./sidenav.po";
import { PreferencesPage } from "./preferences.po";
import { changeSelectValue, scrollPageToTop } from "./util.po";

/**
 * Clone calculators
 */
describe("Solveur - ", () => {
    let listPage: ListPage;
    let calcPage: CalculatorPage;
    let navbar: Navbar;
    let sidenav: SideNav;
    let prefPage: PreferencesPage;

    beforeAll(async () => {
        prefPage = new PreferencesPage();
        listPage = new ListPage();
        calcPage = new CalculatorPage();
        navbar = new Navbar();
        sidenav = new SideNav();
    });

    beforeEach(async () => {
        // disable evil option "empty fields on module creation"
        await prefPage.setEmptyFields(false);
        // force language to prevent issues due to default browser language
        await prefPage.changeLanguage(1); // fr
        await browser.sleep(200);
        await navbar.clickNewCalculatorButton();
    });

    it("load > calculate", async () => {
        await navbar.clickMenuButton();
        await browser.sleep(200);

        await sidenav.clickLoadSessionButton();
        await browser.sleep(200);

        await sidenav.loadSessionFile("./session/session-solveur-chutes.json");
        await browser.sleep(200);

        expect(await navbar.getAllCalculatorTabs().count()).toBe(4);
        await navbar.clickCalculatorTab(3); // n°3 should be the latest

        // check input values
        expect(await calcPage.getInputById("Xinit").getAttribute("value")).toBe("0.5");
        expect(await calcPage.getInputById("Ytarget").getAttribute("value")).toBe("252");
        // check Nub to calculate
        const ntc = calcPage.getSelectById("select_target_nub");
        const ntcV = await calcPage.getSelectValueText(ntc);
        expect(ntcV).toContain("PAB : puissance / Puissance dissipée (PV)");
        // check targetted result
        const ntt = calcPage.getSelectById("select_target_result");
        const nttV = await calcPage.getSelectValueText(ntt);
        expect(nttV).toContain("Puissance dissipée (PV)");
        // check searched Parameter
        const sp = calcPage.getSelectById("select_searched_param");
        const spV = await calcPage.getSelectValueText(sp);
        expect(spV).toContain("Z2 - Cote aval (PAB : chute)");

        // check that "compute" button is active
        const calcButton = calcPage.checkCalcButtonEnabled(true);
        // click "compute" button
        await calcButton.click();
        // check that result is not empty
        const hasResults = await calcPage.hasResults();
        expect(hasResults).toBe(true);

        // change targetted Nub, check that targetted result changes too
        await changeSelectValue(ntc, 0);
        const nttV2 = await calcPage.getSelectValueText(ntt);
        expect(nttV2).not.toContain("Puissance dissipée (PV)");
    });

    it("create > feed > calculate > clone > calculate clone", async () => {
        // 1. create empty Solveur
        await listPage.clickMenuEntryForCalcType(22); // Solveur
        await browser.sleep(500);

        // 2. create PAB:Chute, PAB:Nombre and PAB:Puissance linked to one another
        await navbar.clickNewCalculatorButton();
        await listPage.clickMenuEntryForCalcType(12); // PAB:Chute
        await browser.sleep(500);
        await navbar.clickNewCalculatorButton();

        await listPage.clickMenuEntryForCalcType(13); // PAB:Nombre
        await browser.sleep(500);
        // link DHT to PAB:Chute.DH
        const dht = calcPage.getInputById("DHT");
        await calcPage.setParamMode(dht, "link");
        // Calculate DH
        const dh_nombre = calcPage.getInputById("DH");
        await calcPage.setParamMode(dh_nombre, "cal");

        await navbar.clickNewCalculatorButton();
        await listPage.clickMenuEntryForCalcType(6); // PAB:Puissance
        await browser.sleep(500);
        // link DH to PAB:Nombre.DH
        const dh_puiss = calcPage.getInputById("DH");
        await calcPage.setParamMode(dh_puiss, "link");

        // Go back to Solveur
        await navbar.clickCalculatorTab(0);

        await changeSelectValue(calcPage.getSelectById("select_target_nub"), 1); // "Puissance / PV"
        await browser.sleep(500);
        await changeSelectValue(calcPage.getSelectById("select_searched_param"), 2); // "Chute / Z2"
        await browser.sleep(500);
        await calcPage.getInputById("Ytarget").sendKeys("318");

        // check that "compute" button is active
        const calcButton = calcPage.checkCalcButtonEnabled(true);
        // click "compute" button
        await calcButton.click();
        // check that result is not empty
        const hasResults = await calcPage.hasResults();
        expect(hasResults).toBe(true);

        await scrollPageToTop(); // otherwise clickCloneCalcButton() fails with "Element is not clickable at point"

        await calcPage.clickCloneCalcButton();
        await browser.sleep(500);

        // 4. check existence of the cloned module
        expect(await navbar.getAllCalculatorTabs().count()).toBe(5);
        await navbar.clickCalculatorTab(4); // n°4 should be the latest

        // check that result is empty
        const hasResultsClone1 = await calcPage.hasResults();
        expect(hasResultsClone1).toBe(false);

        // check that "compute" button is active
        const calcButtonClone = calcPage.checkCalcButtonEnabled(true);
        // click "compute" button
        await calcButtonClone.click();
        // check that result is not empty
        const hasResultsClone2 = await calcPage.hasResults();
        expect(hasResultsClone2).toBe(true);
    });

    it("channel flow example > solver > change searched parameter > run calculation", async () => {
        // open "channel flow with hydraulic structures" example
        const examples = await element.all(by.css("#examples-list .load-example"));
        await examples[1].click();
        await browser.sleep(500);

        // select solver tab
        await navbar.clickCalculatorTab(3);
        await browser.sleep(500);

        // modify searched parameter
        const sel = calcPage.getSelectById("select_searched_param");
        await changeSelectValue(sel, 11);
        await browser.sleep(300);
        const selText = await calcPage.getSelectValueText(sel);

        // run calculation
        const calcButton = calcPage.getCalculateButton();
        await calcButton.click();
        await browser.sleep(500);

        // check "search parameter" value has not changed
        expect(await calcPage.getSelectValueText(sel)).toEqual(selText);
    });

    it("check solver searched parameter is set to 'bottom slope'", async () => {
        // open "canal critical slope" example
        const examples = await element.all(by.css("#examples-list .load-example"));
        await examples[3].click();
        await browser.sleep(500);

        // select solver tab
        await navbar.clickCalculatorTab(2);
        await browser.sleep(500);

        // check selected searched parameter text
        const sel = calcPage.getSelectById("select_searched_param");
        const selText = await calcPage.getSelectValueText(sel);
        expect(selText).toEqual("If - Pente du fond (Sec. param.)");
    });
});

/**
 * solver with empty fields option
 */
describe("Solveur - nghyd#601 with empty fields option", () => {
    let prefPage: PreferencesPage;
    let navBar: Navbar;
    let listPage: ListPage;
    let calcPage: CalculatorPage;

    beforeAll(async () => {
        prefPage = new PreferencesPage();
        navBar = new Navbar();
        listPage = new ListPage();
        calcPage = new CalculatorPage();
    });

    async function openCalculator(id: number) {
        await navBar.clickNewCalculatorButton();
        await browser.sleep(200);

        await listPage.clickMenuEntryForCalcType(id);
        await browser.sleep(200);
    }

    async function openSolver() {
        await openCalculator(22);
    }

    beforeEach(async () => {
        // enable evil option "empty fields on module creation"
        await prefPage.setEmptyFields(true);
    });

    it("check solver with empty fields option does not fill inputs - solver alone", async () => {
        // open new solver calculator
        await openSolver();

        // check inputs are empty
        await calcPage.checkEmptyInput("Ytarget");
        await browser.sleep(200);

        await calcPage.checkEmptyInput("Xinit");
        await browser.sleep(200);
    });

    it("check solver with empty fields option does not fill target parameter input", async () => {
        // open "parallel structures" calculator
        await openCalculator(8);

        // open second "parallel structures" calculator
        await openCalculator(8);

        // link Q to first calculator's
        const inpQ = calcPage.getInputById("Q");
        await calcPage.setParamMode(inpQ, "link");
        await browser.sleep(200);

        // open new solver calculator
        await openSolver();

        // check target parameter input is empty
        await calcPage.checkEmptyInput("Ytarget");
        await browser.sleep(200);

        // check initial value input is not empty
        await calcPage.checkEmptyInput("Xinit");
        await browser.sleep(200);
    });

    it("check removing and recreating solver with empty fields option does not fill target parameter input", async () => {
        await navBar.clickNewCalculatorButton();
        await browser.sleep(200);

        // open "channel flow with hydraulic structures" example
        const examples = await element.all(by.css("#examples-list .load-example"));
        await examples[1].click();
        await browser.sleep(500);

        // close existing "solver" calculator
        await navBar.middleClickCalculatorTab(3);
        await browser.sleep(500);

        // open new solver calculator
        await openSolver();

        // check target parameter input is empty
        await calcPage.checkEmptyInput("Ytarget");
        await browser.sleep(200);

        // check initial value input is not empty
        await calcPage.checkEmptyInput("Xinit", false);
        await browser.sleep(200);
    });
});

/**
 * solver without empty fields option
 */
describe("Solveur - nghyd#601 without empty fields option", () => {
    let prefPage: PreferencesPage;
    let navBar: Navbar;
    let listPage: ListPage;
    let calcPage: CalculatorPage;

    beforeAll(async () => {
        prefPage = new PreferencesPage();
        navBar = new Navbar();
        listPage = new ListPage();
        calcPage = new CalculatorPage();
    });

    async function openCalculator(id: number) {
        await navBar.clickNewCalculatorButton();
        await browser.sleep(200);

        await listPage.clickMenuEntryForCalcType(id);
        await browser.sleep(200);
    }

    async function openSolver() {
        await openCalculator(22);
    }

    beforeEach(async () => {
        await browser.manage().window().setPosition(2000, 30);

        // disable evil option "empty fields on module creation"
        await prefPage.setEmptyFields(false);
    });

    it("check solver without empty fields option does not fill inputs - solver alone", async () => {
        // open new solver calculator
        await openSolver();

        // check inputs are empty
        await calcPage.checkEmptyInput("Ytarget");
        await browser.sleep(200);

        await calcPage.checkEmptyInput("Xinit");
        await browser.sleep(200);
    });

    it("check solver without empty fields option fills inputs", async () => {
        // open "parallel structures" calculator
        await openCalculator(8);

        // open second "parallel structures" calculator
        await openCalculator(8);

        // link Q to first calculator's
        const inpQ = calcPage.getInputById("Q");
        await calcPage.setParamMode(inpQ, "link");
        await browser.sleep(200);

        // open new solver calculator
        await openSolver();

        // check target parameter input is not empty
        await calcPage.checkEmptyInput("Ytarget", false);
        await browser.sleep(200);

        // check initial value input is not empty
        await calcPage.checkEmptyInput("Xinit", false);
        await browser.sleep(200);
    });

    it("check removing and recreating solver without empty fields option fills inputs", async () => {
        await navBar.clickNewCalculatorButton();
        await browser.sleep(200);

        // open "channel flow with hydraulic structures" example
        const examples = await element.all(by.css("#examples-list .load-example"));
        await examples[1].click();
        await browser.sleep(500);

        // close existing "solver" calculator
        await navBar.middleClickCalculatorTab(3);
        await browser.sleep(500);

        // open new solver calculator
        await openSolver();

        // check target parameter input is not empty
        await calcPage.checkEmptyInput("Ytarget", false);
        await browser.sleep(200);

        // check initial value input is not empty
        await calcPage.checkEmptyInput("Xinit", false);
        await browser.sleep(200);
    });

    it(" with empty fields option, check selecting a target module does not fill inputs", async () => {
        // enable evil option "empty fields on module creation"
        await prefPage.setEmptyFields(true);
        await browser.sleep(200);

        await navBar.clickNewCalculatorButton();
        await browser.sleep(200);

        // open "channel flow with hydraulic structures" example
        const examples = await element.all(by.css("#examples-list .load-example"));
        await examples[1].click();
        await browser.sleep(500);

        // close existing "solver" calculator
        await navBar.middleClickCalculatorTab(3);
        await browser.sleep(500);

        // open new solver calculator
        await openSolver();
        await browser.sleep(500);

        // select other target module
        const ntc = calcPage.getSelectById("select_target_nub");
        await changeSelectValue(ntc, 1);

        // check target value input is empty
        await calcPage.checkEmptyInput("Ytarget", true);
        await browser.sleep(200);
    });
});
