import { ListPage } from "./list.po";
import { Navbar } from "./navbar.po";
import { browser, by } from "protractor";
import { CalculatorPage } from "./calculator.po";
import { PreferencesPage } from "./preferences.po";
import { SideNav } from "./sidenav.po";

describe("Check translation", () => {
    let listPage: ListPage;
    let navBar: Navbar;
    let calcPage: CalculatorPage;
    let prefPage: PreferencesPage;
    let sideNav: SideNav;

    beforeAll(async () => {
        listPage = new ListPage();
        navBar = new Navbar();
        calcPage = new CalculatorPage();
        prefPage = new PreferencesPage();
        sideNav = new SideNav();
    });

    beforeEach(async () => {
        prefPage.setEmptyFields(false);
    });

    it("variables in results", async () => {
        // *** results in french ***

        prefPage.changeLanguage(1); // fr
        await browser.sleep(200);

        // open "fish ladder: fall" calculator
        await navBar.clickNewCalculatorButton();
        await listPage.clickMenuEntryForCalcType(12);
        await browser.sleep(200);

        // set Z2 to variated mode
        const inpZ2 = calcPage.getInputById("Z2");
        await calcPage.setParamMode(inpZ2, "var");

        // run calculation
        await calcPage.getCalculateButton().click();
        await browser.sleep(500);

        // "variable for X axis" select label
        const selXaxis = calcPage.getSelectById("selectX");
        expect(await calcPage.getMatselectCurrentOptionText(selXaxis)).toEqual("Cote aval");

        // "variable for Y axis" select label
        const selYaxis = calcPage.getSelectById("selectY");
        expect(await calcPage.getMatselectCurrentOptionText(selYaxis)).toEqual("DH : Chute (m)");

        // fixed results variables
        const frr = calcPage.getAllFixedResultsRows();
        let lbl1 = await frr.all(by.css("td")).get(0).getText();
        expect(lbl1).toEqual("Cote amont (m)");

        // variated results headers
        const vrh = calcPage.getAllVariatedResultsTableHeaders();
        let lbl2 = await vrh.get(0).getText();
        expect(lbl2).toEqual("Cote aval");
        let lbl3 = await vrh.get(1).getText();
        expect(lbl3).toEqual("Chute (m)");

        // *** results in english ***

        // setup -> english
        await navBar.clickMenuButton();
        await browser.sleep(200);
        const setupBtn = await sideNav.getSetupButton();
        await setupBtn.click();
        await browser.sleep(200);
        await prefPage.changeLanguage(0); // en
        await browser.sleep(200);

        // back to calculator
        await navBar.clickCalculatorTab(0);
        await browser.sleep(200);

        // "variable for X axis" select label
        expect(await calcPage.getMatselectCurrentOptionText(selXaxis)).toEqual("Downstream elevation");

        // "variable for Y axis" select label
        expect(await calcPage.getMatselectCurrentOptionText(selYaxis)).toEqual("DH : Fall (m)");

        // fixed results variables
        lbl1 = await frr.all(by.css("td")).get(0).getText();
        expect(lbl1).toEqual("Upstream elevation (m)");

        // variated results headers
        lbl2 = await vrh.get(0).getText();
        expect(lbl2).toEqual("Downstream elevation");
        lbl3 = await vrh.get(1).getText();
        expect(lbl3).toEqual("Fall (m)");
    });
});
